# chrono - agendas system
# Copyright (C) 2022  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.postgres.fields import JSONField
from django.db import models
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _

from chrono.utils.misc import generate_slug


class CriteriaCategory(models.Model):
    label = models.CharField(_('Label'), max_length=150)
    slug = models.SlugField(_('Identifier'), max_length=160, unique=True)

    class Meta:
        ordering = ['label']

    def __str__(self):
        return self.label

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = generate_slug(self)
        super().save(*args, **kwargs)

    @property
    def base_slug(self):
        return slugify(self.label)


class Criteria(models.Model):
    category = models.ForeignKey(
        CriteriaCategory, verbose_name=_('Category'), on_delete=models.CASCADE, related_name='criterias'
    )
    label = models.CharField(_('Label'), max_length=150)
    slug = models.SlugField(_('Identifier'), max_length=160)
    condition = models.CharField(_('Condition'), max_length=1000)
    order = models.PositiveIntegerField()

    class Meta:
        ordering = ['order']
        unique_together = ['category', 'slug']

    def __str__(self):
        return self.label

    def save(self, *args, **kwargs):
        if self.order is None:
            max_order = (
                Criteria.objects.filter(category=self.category)
                .aggregate(models.Max('order'))
                .get('order__max')
                or 0
            )
            self.order = max_order + 1
        if not self.slug:
            self.slug = generate_slug(self, category=self.category)
        super().save(*args, **kwargs)

    @property
    def base_slug(self):
        return slugify(self.label)


class Pricing(models.Model):
    label = models.CharField(_('Label'), max_length=150)
    slug = models.SlugField(_('Identifier'), max_length=160, unique=True)
    categories = models.ManyToManyField(
        CriteriaCategory,
        related_name='pricings',
        through='PricingCriteriaCategory',
    )
    criterias = models.ManyToManyField(Criteria)

    class Meta:
        ordering = ['label']

    def __str__(self):
        return self.label

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = generate_slug(self)
        super().save(*args, **kwargs)

    @property
    def base_slug(self):
        return slugify(self.label)


class PricingCriteriaCategory(models.Model):
    pricing = models.ForeignKey(Pricing, on_delete=models.CASCADE)
    category = models.ForeignKey(CriteriaCategory, on_delete=models.CASCADE)
    order = models.PositiveIntegerField()

    class Meta:
        ordering = ['order']
        unique_together = ['pricing', 'category']

    def save(self, *args, **kwargs):
        if self.order is None:
            max_order = (
                PricingCriteriaCategory.objects.filter(pricing=self.pricing)
                .aggregate(models.Max('order'))
                .get('order__max')
                or 0
            )
            self.order = max_order + 1
        super().save(*args, **kwargs)


class AgendaPricing(models.Model):
    agenda = models.ForeignKey('agendas.Agenda', on_delete=models.CASCADE)
    pricing = models.ForeignKey(Pricing, on_delete=models.CASCADE)
    date_start = models.DateField()
    date_end = models.DateField()
    pricing_data = JSONField(null=True)
